"""
interface for user & comandline tool
"""

from colorama import Fore, Back, Style

from .controller import Controller
from .auth_manager import set_auth, remove_pwd

class Scraper:
    """
    docstring for Scraper.
    """

    def __init__(self, url="", name=""):
        self.url = url
        self.name = name.replace(" ", "_").lower()

    def setup(self, username=""):
        if not username or not "." in username:
            print(Fore.BLUE + "ILIAS SCRAPER" + Style.RESET_ALL)
            print(Style.DIM + "You did not specicy a username so please to it now." + Style.RESET_ALL)
            self.username = input(">> Enter your username: ")
        else:
            self.username = username
        set_auth(username)
        self.controller = Controller(self.username)

    def run(self):
        self.controller.init_controller()
        self.controller.download(self.url, name=self.name)

    def remove_password(self, username):
        remove_pwd(username)




import os
import sys
import glob
import click
from colorama import Fore, Back, Style

from .scraper import Scraper
from .auth_manager import remove_pwd

CONFIG_FILE = ".config"
PATH = os.path.join(os.getenv("HOME"), ".iliasScraper")

version_path = os.path.join(os.path.dirname(__file__),"VERSION")

with open(version_path) as f:
    line = f.readline()
    __version__ = line

@click.group()
@click.version_option(__version__)
def main():
    """CLI for the ilias-scraper"""
    if not os.path.exists(PATH):
        os.makedirs(PATH)


@main.command()
@click.option('--url', help='Ilias course url')
@click.option('--username', help='Your Ilias username')
@click.option('--course-name', help='The name of the course, will be used as the name of this scraper')

def create(url, username, course_name):
    """
    Create a new scraper with a url and name
    """
    # url = kwargs.get("url")
    # course_name = kwargs.get("course_name")
    # username = kwargs.get("username")
    scraper_source = f"""
from iliasScraper import scraper
user = '{username}'
sc = scraper.Scraper(url='{url}', name='{course_name}')
sc.setup(user)
sc.run()"""
    course_name = course_name.replace(" ", "_")
    with open(f"{PATH}/{course_name}.py", "w") as scraper_file:
        scraper_file.write(scraper_source)
    print(Fore.GREEN + f"Scraper {course_name} has been created!")


@main.command()
@click.option('--name', help='Name of the scraper', required=True)
@click.option('--fallback-extension', required=False)
def run(name, fallback_extension="txt"):
    """
    Run a previously created scraper. Name can be specified with
    and without the .py ending
    """
    scraper_file = f"{PATH}/{name}"
    if not ".py" in scraper_file:
        scraper_file += ".py"
    if not os.path.exists(scraper_file):
        print(Fore.RED + "The specified name does not match any configured scrapers.")
        print("Use <<iliasScraper list>> to list all current scrapers.")
        return 0
    os.system(f"python3 {scraper_file}")

@main.command()
def list(**kwargs):
    """
    List all existing scraper
    """
    files = [f for f in glob.glob(PATH+"/*.py", recursive=True)]
    if not files:
        print(Fore.RED+"There are no scrapers configured."+Style.RESET_ALL)
        return 0
    print(Fore.BLUE + "Currently configured scrapers:"+Style.RESET_ALL)
    for f in files:
        filename_printable = f.split("/")[-1:][0]
        print(f"- {filename_printable}")


@main.command()
@click.option('--username', required=True)
def remove_password(username):
    """
    Remove the stored password
    """
    remove_pwd(username)
    print(Fore.RED + f"Password for {username} has been removed!")

if __name__ == '__main__':
    args = sys.argv
    if "--help" in args or len(args) == 1:
        print(Fore.BLUE + "ILIAS SCRAPER")
        print(Style.RESET)
        print(Style.DIM + "scraper for ilias platform of of the uni constance")
        print("create a scraper via <create> <url> <title> or run via <run> <name>")
    main()




# -*- coding: utf-8 -*-

import sys
import getpass
import keyring
from .config import SCRAPER_NAME
from colorama import Fore, Back, Style

PASSWORD = ""

def store_pwd(pwd,username):
	keyring.set_password(SCRAPER_NAME, username, pwd)

def read_pwd(username):
	return keyring.get_password(SCRAPER_NAME, username)

def set_auth(username):
	global PASSWORD

	if read_pwd(username) != None:
		PASSWORD = read_pwd(username)
	else:
		print(Fore.RED + "For your first run you have to set a password, you can choose to store it or enter it for every run." + Style.RESET_ALL)
		print(">> Please enter your password: ")
		PASSWORD = getpass.getpass()
		print(">> Would you like to store the password encrpyted for the future? [y/N]")
		should_store = str(sys.stdin.readline()[:-1])
		if should_store == "y":
			store_pwd(PASSWORD, username)

def remove_pwd(username):
	keyring.delete_password(SCRAPER_NAME, username)

def get_pwd():
	return PASSWORD




import requests
from bs4 import BeautifulSoup

class RequestHandler:
    """
    docstring for RequestHandler.
    """

    def __init__(self, session):
        self.parser = "html.parser"
        self.session = session

    def get_soup(self, url):
        try:
            response = self.session.get(url)
        except Exception as err:
            print(f"Error: {err}")
            # raise err
            print(f"WITH FOLLOWING URL: {url=}")
            raise err
            return False
        if response.status_code == 200:
            soup = BeautifulSoup(response.content, self.parser)
            # print(f"OK! Url: {url}")
            return soup
        else:
            print(f'Something went wrong. Got the following response code: {response.status_code}')
            return None

    def get_file(self, url):
        try:
            response = self.session.get(url, allow_redirects=True)
        except Exception as err:
            print(f"Error: {err}")
            print(f"With the following: {url=}")
            raise err
            return False
        if response.status_code == 200:
            return response
        else:
            print(f'Something went wrong. Got the following response code: {response.status_code}')
            return None




import os
import fleep

class DownloadHandler:
    """
    docstring for RequestHandler.
    """

    def __init__(self, request_handler):
        self.request_handler = request_handler

    def download(self, file_dict, path):
        path = os.getcwd() + "/" + path
        if len(path) == 0:
            path = "test"
        self._verify_path(path)
        url = file_dict["url"]
        file_name = file_dict["file_name"]
        file = self.request_handler.get_file(url)
        file_loc = path+"/"+file_name
        open(file_loc, "wb").write(file.content)

        with open(file_loc, "rb") as file:
            info = fleep.get(file.read(256))
        if not info.extension:
            # default extension
            extension = "txt"
        else:
            extension = info.extension[0]
        if extension == "ai":
            extension = "pdf"
        elif extension == "pages":
            extension = "zip"

        os.rename(r""+file_loc,r""+file_loc+"."+extension)

    def _verify_path(self, path):
        folder_list = path.split("/")
        for i in range(0, len(folder_list)):
            sub_path = "/"+"/".join(folder_list[1:i+3])
            if not os.path.exists(sub_path):
                # todo check windows
                original_umask = os.umask(000)
                os.makedirs(sub_path, mode=0o777), #mode = 0o666)
                os.umask(original_umask)
        return 1




import pprint
from tqdm import tqdm
from colorama import Fore, Back, Style

from .login_handler import LoginHandler
from .file_parser import FileParser
from .request_handler import RequestHandler
from .download_handler import DownloadHandler

pp = pprint.PrettyPrinter(indent=4)

class Controller:
    """
    The controller of the package - here comes everything together
    """

    def __init__(self, username):
        self.username = username
        self.login_handler = LoginHandler(self.username)
        self.test_course_url = "https://ilias.uni-konstanz.de/ilias/goto_ilias_uni_crs_1078392.html"
        self.test_file1 = "https://ilias.uni-konstanz.de/ilias/goto_ilias_uni_file_1078407_download.html"
        self.test_file2 = "https://ilias.uni-konstanz.de/ilias/goto_ilias_uni_file_1091580_download.html"

    def collect(self, url):
        file_dict = {}
        print(">> Getting all folder urls...")
        all_urls = self.file_parser.get_all_urls(url)#.append(url)
        all_urls.append(url)
        print(Style.RESET_ALL)
        for link in all_urls:
            soup = self.request_handler.get_soup(link)
            file_dict_for_link = self.file_parser.parse(soup, url=self.test_course_url)
            file_dict.update(file_dict_for_link)
        return file_dict

    def init_controller(self):
        self.session = self.login_handler.login()
        self.request_handler = RequestHandler(self.session)
        self.file_parser = FileParser(self.request_handler)
        self.download_handler = DownloadHandler(self.request_handler)

    def download(self, url, name="course name"):
        print(Fore.BLUE + "\t\t ILIAS SCRAPER")
        print(Fore.GREEN + f" - {name} -")
        print(Style.RESET_ALL)
        files = self.collect(url)
        print(">> Starting download")
        for path in files.keys():
            files_for_path = files[path]
            print(Style.DIM + path)
            for file in files_for_path:
                print(file["file_name"])
                self.download_handler.download(file, path)

    def test(self):
        self.download_handler.download({"url":self.test_file2, "file_name":"testfile"}, "")


# controller = Controller(controller.test_course_url,
# name="Bachelor-Seminar Datenanalyse und Visualisierung | Master's Seminar Data Analysis and Visualization")
# controller.download()
# controller.test()




import re
from tqdm import tqdm
from colorama import Fore, Back, Style

class FileParser:
    """
    docstring for FileParser.
    """

    def __init__(self, request_handler):
        self.parser = 'html.parser'
        self.ignore = ["weblink", "test", "forum", "übung", "inhaltsseite"]
        self.file_id = "datei"
        self.folder_id = "ordner"
        self.current_path = "/"
        self.dir_dict = {}
        self.done_folders = []
        self.folder_todo = []
        self.todo_folders_index = 0
        self.layer = 0
        self.request_handler = request_handler
        self.all_urls = []


    def filter_ignored(self, img_list):
        def filter_(title):
            return any(map(lambda el: el in title.lower(), self.ignore))
        return [tag for tag in img_list if not filter_(tag["title"])]

    def _extract_href(self, img):
        return img.parent["href"]

    def _pathify_folder_name(self, folder_name):
        folder_name = "_".join(folder_name.split(" "))
        folder_name = "_".join(folder_name.split("|"))
        folder_name = "_".join(folder_name.split("'"))
        folder_name = re.sub("_+", "_", folder_name)
        return folder_name

    def _extract_file_name(self, img):
        return img.parent.parent.parent.find_all("a")[1].text

    def update_dir_dict(self, img_list, path):
        """
        """
        dir_dict = {}
        for img in img_list:
            img_title = img["title"].lower()
            if self.file_id in img_title:
                # todo: split current path and thus create real sub dicts
                if not path in dir_dict:
                    dir_dict[path] = []
                dir_dict[path].append({
                        "url": self._extract_href(img),
                        "file_name": self._extract_file_name(img)
                        })
        return dir_dict

    def get_all_urls(self, url):
        soup = self.request_handler.get_soup(url)
        links = soup.find_all('a')
        if links is None or len(links) == 0 or url in self.all_urls:
            self.all_urls.append(url)
            return 1
        else:
            if "fold" in url:
                self.all_urls.append(url)
                print(Style.DIM + url)
            for link in links:
                if not link.has_attr("href"):
                    continue
                href = link['href']
                if "https" in href and "fold" in href:
                    self.get_all_urls(href)
        return list(set(self.all_urls))

    def extract_path_from_breadcrumb(self, breadcrumb_string):
        total_path_list = breadcrumb_string.split("\n")
        split_index = list(map(lambda item: "Lehrveranstaltung" in item, total_path_list)).index(True)
        path_list = total_path_list[split_index+1:][:-1]
        return path_list

    def parse(self, soup, url="/"):


        breadcrumb_string = soup.find_all("ol", class_="breadcrumb")[0].text
        path = "/".join(list(map(lambda x: self._pathify_folder_name(x),
                        self.extract_path_from_breadcrumb(breadcrumb_string))))


        all_img_tags = soup.find_all("img", class_="ilListItemIcon")
        f_img_tags = self.filter_ignored(all_img_tags)

        dir_dict = self.update_dir_dict(f_img_tags, path)

        return dir_dict




import requests

from .ilias_exceptions import LoginFailedError
from .config import LOGIN_POST_URL
from .auth_manager import get_pwd

class LoginHandler:
    """
    Handle login with
    """

    def __init__(self, username):
        self.payload = {
                "username": username,
                "password": get_pwd(),
                "cmd[doStandardAuthentication]": "Anmelden"
        }
        self.session = requests.Session()

    def login(self):
        with self.session as session_r:
            login_resp = session_r.post(LOGIN_POST_URL, data=self.payload)
            if login_resp.status_code != 200:
                raise LoginFailedError(f"Login failed with status_code {login_resp.status_code}")
        return session_r
